@@ -11,6 +11,10 @@
 #import "Track.h"
 #import "ID3Tagger.h"
 #import "defs.h"
+// for stat
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 // these correspond to the identifiers in the NIB file
 #define COLUMN_TITLE @"Title"
@@ -261,6 +265,15 @@
 		[track setValuesToTrack:trackWithTag];
 		[tagger release];
 	}
+  
+  // don't trust old filesize
+  
+  struct stat sb;
+	// check file exists
+	if (stat([[track fullPath] fileSystemRepresentation], &sb) == -1)
+		return [[NJBTransactionResult alloc] initWithSuccess:NO resultString:NSLocalizedString(@"File no longer exists", nil)];
+
+  [track setFilesize:sb.st_size];
 	
 	return [theNJB uploadTrack:track];
 }
@@ -597,6 +610,7 @@
   // to store the array since we can't enumerate from array controller without sorting
   NSMutableArray *iTunesTracks = [[NSMutableArray alloc] init];
 	
+  NSFileManager *manager = [NSFileManager defaultManager];
 	NSDictionary *thisTrackDict;
 	NSArray *tracksDictArray = [tracksDict allValues];
 	NSEnumerator *objectEnumerator = [tracksDictArray objectEnumerator];
@@ -610,9 +624,19 @@
     if (location == nil)
     {
       NSLog(@"location nil when reading in iTunes library! Ignoring entry");
+      [track release];
       break;
     }
-		[track setFullPath:[[NSURL URLWithString:location] relativePath]];
+    
+    [track setFullPath:[[NSURL URLWithString:location] relativePath]];
+    
+    if (![manager fileExistsAtPath:[track fullPath]])
+    {
+      NSLog(@"iTunes track %@ no longer exists", [track fullPath]);
+      [track release];
+      break;
+    }
+    
 		[track fileTypeFromExtension];
 		
 		[track setTitle:[thisTrackDict valueForKey:@"Name"]];
@@ -622,7 +646,11 @@
 		[track setLength:([[thisTrackDict valueForKey:@"Total Time"] unsignedLongValue] / 1000)];
 		[track setYear:[[thisTrackDict valueForKey:@"Year"] unsignedIntValue]];
 		[track setTrackNumber:[[thisTrackDict valueForKey:@"Track Number"] unsignedIntValue]];
-		[track setFilesize:[[thisTrackDict valueForKey:@"Size"] unsignedLongLongValue]];
+    
+    // don't trust iTunes filesize - it might have changed
+    NSDictionary *attributes = [manager fileAttributesAtPath:[track fullPath] traverseLink:YES];
+    [track setFilesize:[[attributes objectForKey:NSFileSize] unsignedLongLongValue]];
+    
     [track setDateAdded:[thisTrackDict valueForKey:@"Date Added"]];
     [track setRating:[[thisTrackDict valueForKey:@"Rating"] unsignedIntValue]];
 
