@@ -23,7 +23,15 @@
 #include <unistd.h>
 #import "UnicodeWrapper.h"
 #import "Album.h"
-
+#import "QueueTab.h"
+#import "NJBQueueConsumer.h"
+#include "usb.h"
+#include "libusb-glue.h"
+#include "usbi.h"
+#include <CoreFoundation/CoreFoundation.h>
+#include <IOKit/IOKitLib.h>
+#include <IOKit/IOCFPlugIn.h>
+#include <IOKit/usb/IOUSBLib.h>
 #include "ptp.h"
 
 // number of tracks in tracklist downloaded in between progress bar updates (MTP only)
@@ -46,7 +54,9 @@ StatusDisplayer *statusDisplayerGlobal;
 // so we only change it before a transfer, so it is not changed during a transfer
 // which could mess things up
 - (void)enableTurbo;
-
+void deviceAdded(void *refCon, io_iterator_t iterator);
+- (void)initDevice;
+io_iterator_t deviceAddedIter;
 @end
 
 @implementation NJB
@@ -74,6 +84,8 @@ StatusDisplayer *statusDisplayerGlobal;
 		device = NULL;
     
     albums = nil;
+    
+    LIBMTP_Init();
 	}
 	return self;
 }
@@ -241,6 +253,11 @@ StatusDisplayer *statusDisplayerGlobal;
   }
 }
 
+/*// TEST
+#include "libmtp/src/ptp-pack.c"
+// !TEST*/
+
+
 /* connect to the NJB
  */
 - (NJBTransactionResult *)connect
@@ -295,7 +312,6 @@ StatusDisplayer *statusDisplayerGlobal;
 	  
     //usb_set_debug(9);
     
-		LIBMTP_Init();
 		device = LIBMTP_Get_First_Device();
     
 		if (device == NULL)
@@ -322,89 +338,19 @@ StatusDisplayer *statusDisplayerGlobal;
     return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:errorString extendedErrorString:[self njbErrorString]] autorelease];
 	}
 	
-	[self updateDiskSpace];
-	
-	// enable UTF8
-	if (!mtpDevice)
-		NJB_Set_Unicode(NJB_UC_UTF8);
-	
-	// check we have reset the cache, should have been done on disconnect
-	[cachedTrackList release];
-	cachedTrackList = nil;
-	[cachedDataFileList release];
-	cachedDataFileList = nil;
-	
-	if (mtpDevice)
-	{
-		char *tempString =  LIBMTP_Get_Modelname(device);
-    if (tempString != NULL)
-    {
-      deviceString = [NSString stringWithUTF8String:tempString];
-      free(tempString);
-    }
-
-		tempString = LIBMTP_Get_Serialnumber(device);
-		if (tempString != NULL)
-    {
-      deviceIDString = [NSString stringWithUTF8String:tempString];
-      free(tempString);
-    }
-
-		tempString = LIBMTP_Get_Deviceversion(device);
-    NSString *versionString = nil;
-    if (tempString != NULL)
-    {
-      versionString = [NSString stringWithUTF8String:tempString];
-      free(tempString);
-    }
-		
-		NSArray *versions = [versionString componentsSeparatedByString:@"_"];
-		if ([versions count] != 2)
-		{
-			deviceVersionString = versionString;
-			firmwareVersionString = @"";
-		}
-		else
-		{
-			deviceVersionString = [versions objectAtIndex:1];
-			firmwareVersionString = [versions objectAtIndex:0];
-		}
-		
-		uint8_t currentLevel = 0;
-		if (LIBMTP_Get_Batterylevel(device, &batteryLevelMax, &currentLevel) != 0)
-		{
-			NSLog(@"Could not get battery level property description");
-      NSString *errorString = [self njbErrorString];
-      NSLog(errorString);
-      //return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:@"Could not get battery level property description" extendedErrorString:errorString] autorelease];
-		}
-		
-		LIBMTP_Get_Supported_Filetypes(device, &supportedFiletypes, &supportedFiletypesLength);
-    
-    [self downloadAlbumList];
-		
-		// download the data files to the cache
-		//[self dataFiles];
-	}
-	else
-	{
-		[self storeDeviceString];
-		[self storeFirmwareVersionString];
-		[self storeDeviceIDString];
-		[self storeDeviceVersionString];
-	}
-
+	[self initDevice];
+  
 	// send notification, make sure this is last so all variables are set above
 	[statusDisplayer postNotificationName:NOTIFICATION_CONNECTED object:self];
   
   
- /* 
+ /*
   // TEST to read in objectproplist data from a file to debug
   
   NSLog(@"Starting test");
 
   //NSString *filename = @"/Users/richard/njb/sansa-e280-crash.log";
-  NSString *filename = @"/Users/richard/njb/sansa-e260-crash.log";
+  NSString *filename = @"/Users/richard/njb/opl-crash-20081027/session-with-usb-bulk-debugging.txt";
 
   NSFileManager *fileManager = [[NSFileManager alloc] init];
 	NSDictionary *fileAttributes = [fileManager fileAttributesAtPath:filename traverseLink:YES];
@@ -2404,6 +2350,540 @@ int mtp_progress(uint64_t const sent, uint64_t const total, void const * const d
   cancelCurrentTransaction = YES;
 }
 
+// all the stuff that follows is extremely experimental code to try to
+// get notifications of when devices are connected. It doesn't yet work.
+
+- (void)runNotificationLoopWithObject:(id)obj
+{
+  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
+  
+  NSArray *objectArray = (NSArray *)obj;
+  QueueTab *queueTab = [objectArray objectAtIndex:0];
+  NJBQueueConsumer *njbQueue = [objectArray objectAtIndex:1];
+  
+  IONotificationPortRef	notifyPort;
+  mach_port_t masterPort;
+  CFMutableDictionaryRef matchingDict;
+  CFRunLoopSourceRef runLoopSource;
+  kern_return_t ret;
+
+  // first create a master_port for my task
+  ret = IOMasterPort(MACH_PORT_NULL, &masterPort);
+  if (ret || !masterPort)
+  {
+    NSLog(@"runNotificationLoop: couldn't create a master IOKit Port(%08x)", ret);
+    return ;
+  }
+
+  // Set up the matching criteria for the devices we're interested in
+  matchingDict = IOServiceMatching(kIOUSBDeviceClassName);	// Interested in instances of class IOUSBDevice and its subclasses
+  if (!matchingDict)
+  {
+    NSLog(@"runNotificationLoop: can't create a USB matching dictionary");
+    // TODO: do we need this?
+    //mach_port_deallocate(mach_task_self(), masterPort);
+    return;
+  }
+    
+  // Create a notification port and add its run loop event source to our run loop
+  // This is how async notifications get set up.
+  notifyPort = IONotificationPortCreate(masterPort);
+  runLoopSource = IONotificationPortGetRunLoopSource(notifyPort);
+    
+  CFRunLoopAddSource(CFRunLoopGetCurrent(), runLoopSource, kCFRunLoopDefaultMode);
+    
+  // Retain additional references because we use this same dictionary with two calls to 
+  // IOServiceAddMatchingNotification, each of which consumes one reference.
+  // NEED TO ADD THIS BACK IN IF ADD DEVICE REMOVED NOTIFICATION
+//  matchingDict = (CFMutableDictionaryRef) CFRetain(matchingDict); 
+  
+  // this is all the stuff the callback needs
+  NSArray *callbackArray = [NSArray arrayWithObjects:self, njbQueue, queueTab, nil];
+    
+  // Now set up two notifications, one to be called when a raw device is first matched by I/O Kit, and the other to be
+  // called when the device is terminated.
+  ret = IOServiceAddMatchingNotification(notifyPort,
+                                         kIOFirstMatchNotification,
+                                         matchingDict,
+                                         deviceAdded,
+                                         callbackArray,
+                                         &deviceAddedIter);
+                                         
+  NSLog(@"IOServiceAddMatchingNotification returned %d", ret);
+  
+  io_service_t usbDevice;
+  while (usbDevice = IOIteratorNext(deviceAddedIter))
+  {
+    ret = IOObjectRelease(usbDevice);
+  }
+  
+  /*ret = IOServiceAddMatchingNotification(notifyPort,
+                                        kIOTerminatedNotification,
+                                        matchingDict,
+                                        deviceRemoved,
+                                        self,
+                                        &deviceRemovedIter);*/
+                                            
+  // Now done with the master_port
+  //mach_port_deallocate(mach_task_self(), masterPort);
+  masterPort = 0;
+
+  // Start the run loop. Now we'll receive notifications.
+  CFRunLoopRun();
+  
+  // never get here
+  NSLog(@"Got to end of notification run loop!");
+  [pool release];
+}
+
+void deviceAdded(void *refCon, io_iterator_t iterator)
+{
+  NSLog(@"deviceAdded");
+  
+  if (iterator != deviceAddedIter)
+    NSLog(@"iterator != deviceAddedIter!");
+    
+  NSArray *callbackArray = (NSArray *)refCon;
+  NJB *theNJB = [callbackArray objectAtIndex:0];
+  // don't connect if already connected
+  if ([theNJB isConnected])
+    return;
+  NJBQueueConsumer *njbQueue = [callbackArray objectAtIndex:1];
+  QueueTab *queueTab = [callbackArray objectAtIndex:2];
+  
+  NJBQueueItem *connect = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(connectWithIterator)];
+	[connect setStatus:STATUS_CONNECTING];
+  //[connect setDisplayStatus:NO];
+	
+  // TODO: do this in the GUI thread
+	[queueTab addItem:connect withSubItems:nil withDescription:NSLocalizedString(@"Connecting to NJB", nil)];
+	
+	[[njbQueue lock] lock];
+	[[njbQueue queue] addObject:connect];
+	[[njbQueue lock] unlockWithCondition:([[njbQueue queue] isEmpty] ? NO_QUEUE_ITEMS : SOME_QUEUE_ITEMS)];
+  
+  [connect release];
+}
+
+// runs in NJB thread
+- (NJBTransactionResult *)connectWithIterator
+{
+  kern_return_t	result;
+  io_service_t usbDevice;
+  IOCFPlugInInterface **plugInInterface = NULL;
+  SInt32 score;
+  struct usb_device *dev;
+  uint16_t vendor;
+  uint16_t product;
+
+  IOUSBDeviceInterface **deviceInterface = NULL;
+  
+  u_int16_t address = 0;
+  u_int32_t location;
+
+  /* for use in retrieving device description */
+  IOUSBDevRequest req;
+  
+  req.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBStandard, kUSBDevice);
+  req.bRequest = kUSBRqGetDescriptor;
+  req.wValue = kUSBDeviceDesc << 8;
+  req.wIndex = 0;
+  req.wLength = sizeof(IOUSBDeviceDescriptor);
+  
+  const LIBMTP_device_entry_t mtp_device_table[] = {
+  #include "music-players.h"
+  };
+  const int mtp_device_table_size = sizeof(mtp_device_table) / sizeof(LIBMTP_device_entry_t);
+
+  while (usbDevice = IOIteratorNext(deviceAddedIter))
+  {
+    // only go round until we've found something to connect to
+    // but still need to free other usbDevices
+    
+    dev = malloc(sizeof(*dev));
+    // TODO: check dev != NULL
+    memset((void *)dev, 0, sizeof(*dev));
+    
+    dev->next = NULL;
+    dev->prev = NULL;
+    dev->bus = NULL;
+    dev->num_children = 0;
+    dev->children = NULL;
+    
+    /* Create an intermediate plug-in */
+    result = IOCreatePlugInInterfaceForService(usbDevice, kIOUSBDeviceUserClientTypeID,
+                                               kIOCFPlugInInterfaceID, &plugInInterface,
+                                               &score);
+
+    result = IOObjectRelease(usbDevice);
+    if (result || !plugInInterface)
+      continue;
+
+    (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID),
+                                       (LPVOID)&deviceInterface);
+
+    /* done with this */
+    (*plugInInterface)->Stop(plugInInterface);
+    IODestroyPlugInInterface (plugInInterface);
+    plugInInterface = NULL;
+
+    if (!deviceInterface)
+      continue;
+      
+    //result = (*deviceInterface)->GetDeviceVendor(deviceInterface, &vendor);
+    //result = (*deviceInterface)->GetDeviceProduct(deviceInterface, &product);
+    //NSLog(@"Found device %04x/%04x", vendor, product);
+    
+
+    req.pData = &(dev->descriptor);
+    result = (*(deviceInterface))->DeviceRequest(deviceInterface, &req);
+
+    USB_LE16_TO_CPU(dev->descriptor.bcdUSB);
+    USB_LE16_TO_CPU(dev->descriptor.idVendor);
+    USB_LE16_TO_CPU(dev->descriptor.idProduct);
+    USB_LE16_TO_CPU(dev->descriptor.bcdDevice);
+
+///    result = (*(deviceInterface))->GetDeviceAddress(deviceInterface, (USBDeviceAddress *)&address);
+    result = (*(deviceInterface))->GetLocationID(deviceInterface, &location);
+
+    sprintf(dev->filename, "%03i-%04x-%04x-%02x-%02x", address,
+      dev->descriptor.idVendor, dev->descriptor.idProduct,
+      dev->descriptor.bDeviceClass, dev->descriptor.bDeviceSubClass);
+      
+    NSLog([NSString stringWithCString:dev->filename]);
+
+    dev->dev = (USBDeviceAddress *)malloc(4);
+    memcpy(dev->dev, &location, 4);
+
+    /* release the device now */
+    (*(deviceInterface))->Release(deviceInterface);
+    
+    // see if we should connect to it
+    int i;
+    int found = 0;
+
+    // First check if we know about the device already.
+    // Devices well known to us will not have their descriptors
+    // probed, it caused problems with some devices.
+    for (i = 0; i < mtp_device_table_size; i++)
+    {
+      if (dev->descriptor.idVendor == mtp_device_table[i].vendor_id && dev->descriptor.idProduct == mtp_device_table[i].product_id)
+      {
+        NSLog(@"found device");
+        found = 1;
+        break;
+      }
+    }
+    
+      // get config - must be done before doing anything with dev
+      usb_dev_handle *devh;
+
+      devh = usb_open(dev);
+      if (devh) {
+        usb_fetch_and_parse_descriptors(devh);
+
+        /////////usb_close(devh);
+      }
+
+    //result = probe_device_descriptor(dev, NULL);
+    //NSLog(@"probe device descriptor returned %d", result);
+    
+    
+    
+  unsigned char buf[1024], cmd;
+ // int i;
+  int ret;
+    
+
+  /*
+   * Loop over the device configurations and interfaces. Nokia MTP-capable 
+   * handsets (possibly others) typically have the string "MTP" in their 
+   * MTP interface descriptions, that's how they can be detected, before
+   * we try the more esoteric "OS descriptors" (below).
+   */
+  for (i = 0; i < dev->descriptor.bNumConfigurations; i++) {
+    uint8_t j;
+    
+    for (j = 0; j < dev->config[i].bNumInterfaces; j++) {
+      int k;
+      for (k = 0; k < dev->config[i].interface[j].num_altsetting; k++) {
+        buf[0] = '\0';
+        ret = usb_get_string_simple(devh, 
+				    dev->config[i].interface[j].altsetting[k].iInterface, 
+				    (char *) buf, 
+				    1024);
+	if (ret < 3)
+	  continue;
+        if (strcmp((char *) buf, "MTP") == 0) {
+	        usb_close(devh);
+          NSLog(@"is MTP");
+        }
+      }
+    }
+  }
+  
+  /* Read the special descriptor */
+  ret = usb_get_descriptor(devh, 0x03, 0xee, buf, sizeof(buf));
+
+  
+  /* Check if descriptor length is at least 10 bytes */
+  if (ret < 10) {
+    NSLog(@"ret < 10");
+    usb_close(devh);
+//    return 0;
+  }
+      
+  /* Check if this device has a Microsoft Descriptor */
+  if (!((buf[2] == 'M') && (buf[4] == 'S') &&
+	(buf[6] == 'F') && (buf[8] == 'T'))) {
+    NSLog(@"MSFT");
+    usb_close(devh);
+//    return 0;
+  }
+      
+  /* Check if device responds to control message 1 or if there is an error */
+  cmd = buf[16];
+  ret = usb_control_msg (devh,
+			 USB_ENDPOINT_IN|USB_RECIP_DEVICE|USB_TYPE_VENDOR,
+			 cmd,
+			 0,
+			 4,
+			 (char *) buf,
+			 sizeof(buf),
+                         10000);
+
+  /* If this is true, the device either isn't MTP or there was an error */
+  if (ret <= 0x15) {
+    /* TODO: If there was an error, flag it and let the user know somehow */
+    /* if(ret == -1) {} */
+    usb_close(devh);
+    NSLog(@"ret <= 0x15");
+//    return 0;
+  }
+  
+  /* Check if device is MTP or if it is something like a USB Mass Storage 
+     device with Janus DRM support */
+  if ((buf[0x12] != 'M') || (buf[0x13] != 'T') || (buf[0x14] != 'P')) {
+    usb_close(devh);
+    NSLog(@"not MTP");
+//    return 0;
+  }
+      
+  /* After this point we are probably dealing with an MTP device */
+
+  /* Check if device responds to control message 2 or if there is an error*/
+  ret = usb_control_msg (devh,
+			 USB_ENDPOINT_IN|USB_RECIP_DEVICE|USB_TYPE_VENDOR,
+			 cmd,
+			 0,
+			 5,
+			 (char *) buf,
+			 sizeof(buf),
+                         10000);
+
+  
+  /* If this is true, the device errored against control message 2 */
+  if (ret == -1) {
+    /* TODO: Implement callback function to let managing program know there
+       was a problem, along with description of the problem */
+    fprintf(stderr, "Potential MTP Device with VendorID:%04x and "
+	    "ProductID:%04x encountered an error responding to "
+	    "control message 2.\n"
+	    "Problems may arrise but continuing\n",
+	    dev->descriptor.idVendor, dev->descriptor.idProduct);
+  } else if (ret <= 0x15) {
+    /* TODO: Implement callback function to let managing program know there
+       was a problem, along with description of the problem */
+    fprintf(stderr, "Potential MTP Device with VendorID:%04x and "
+	    "ProductID:%04x responded to control message 2 with a "
+	    "response that was too short. Problems may arrise but "
+	    "continuing\n",
+	    dev->descriptor.idVendor, dev->descriptor.idProduct);
+  } else if ((buf[0x12] != 'M') || (buf[0x13] != 'T') || (buf[0x14] != 'P')) {
+    /* TODO: Implement callback function to let managing program know there
+       was a problem, along with description of the problem */
+    fprintf(stderr, "Potential MTP Device with VendorID:%04x and "
+	    "ProductID:%04x encountered an error responding to "
+	    "control message 2\n"
+	    "Problems may arrise but continuing\n",
+	    dev->descriptor.idVendor, dev->descriptor.idProduct);
+  }
+  
+  /* Close the USB device handle */
+  usb_close(devh);
+  
+  NSLog(@"got to end of probe descriptor: is MTP device");
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    
+    // If we didn't know it, try probing the "OS Descriptor".
+    if (!found) {
+      NSLog(@"unkown device");
+      // TODO: probe_device_descriptor is static in libusb-glue.c
+      //if (probe_device_descriptor(dev, NULL))
+        //found = 1;
+    }
+    
+    if (found)
+    {        
+      // Conjure a device list
+      LIBMTP_raw_device_t *rawDevice = (LIBMTP_raw_device_t *) malloc(sizeof(LIBMTP_raw_device_t));
+      if (rawDevice == NULL) {
+        // Out of memory
+        return nil;
+      }
+
+      // Assign default device info
+      rawDevice->device_entry.vendor = NULL;
+      rawDevice->device_entry.vendor_id = dev->descriptor.idVendor;
+      rawDevice->device_entry.product = NULL;
+      rawDevice->device_entry.product_id = dev->descriptor.idProduct;
+      rawDevice->device_entry.device_flags = 0x00000000U;
+      // See if we can locate some additional vendor info and device flags
+      int j = 0;
+      for (j = 0; j < mtp_device_table_size; j++)
+      {
+        if(dev->descriptor.idVendor == mtp_device_table[j].vendor_id && dev->descriptor.idProduct == mtp_device_table[j].product_id)
+        {
+          rawDevice->device_entry.vendor = mtp_device_table[j].vendor;
+          rawDevice->device_entry.product = mtp_device_table[j].product;
+          rawDevice->device_entry.device_flags = mtp_device_table[j].device_flags;
+          break;
+        }
+      }
+      
+      rawDevice->devnum = dev->devnum;
+      
+ ///     rawDevice->libusb_device = dev;
+      
+      device = LIBMTP_Open_Raw_Device(rawDevice);
+      if (device == NULL)
+      {
+        // free other stuff and return error
+        return;
+      }
+      
+      connected = YES;
+			[self setStatus:STATUS_CONNECTED];
+      
+      // TODO: free raw device
+      
+      [self initDevice];
+      [statusDisplayer postNotificationName:NOTIFICATION_CONNECTED object:self];
+      
+      return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
+      
+      // TODO: free other usbDevices
+      
+    }
+    else
+    {
+      if (dev->dev != NULL)
+        free(dev->dev);
+        free(dev);
+    }
+  }
+}
+
+- (void)initDevice
+{
+  [self updateDiskSpace];
+	
+	// enable UTF8
+	if (!mtpDevice)
+		NJB_Set_Unicode(NJB_UC_UTF8);
+	
+	// check we have reset the cache, should have been done on disconnect
+	[cachedTrackList release];
+	cachedTrackList = nil;
+	[cachedDataFileList release];
+	cachedDataFileList = nil;
+	
+	if (mtpDevice)
+	{
+		char *tempString =  LIBMTP_Get_Modelname(device);
+    if (tempString != NULL)
+    {
+      deviceString = [NSString stringWithUTF8String:tempString];
+      free(tempString);
+    }
+
+		tempString = LIBMTP_Get_Serialnumber(device);
+		if (tempString != NULL)
+    {
+      deviceIDString = [NSString stringWithUTF8String:tempString];
+      free(tempString);
+    }
+
+		tempString = LIBMTP_Get_Deviceversion(device);
+    NSString *versionString = nil;
+    if (tempString != NULL)
+    {
+      versionString = [NSString stringWithUTF8String:tempString];
+      free(tempString);
+    }
+		
+		NSArray *versions = [versionString componentsSeparatedByString:@"_"];
+		if ([versions count] != 2)
+		{
+			deviceVersionString = versionString;
+			firmwareVersionString = @"";
+		}
+		else
+		{
+			deviceVersionString = [versions objectAtIndex:1];
+			firmwareVersionString = [versions objectAtIndex:0];
+		}
+		
+		uint8_t currentLevel = 0;
+		if (LIBMTP_Get_Batterylevel(device, &batteryLevelMax, &currentLevel) != 0)
+		{
+			NSLog(@"Could not get battery level property description");
+      NSString *errorString = [self njbErrorString];
+      NSLog(errorString);
+      //return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:@"Could not get battery level property description" extendedErrorString:errorString] autorelease];
+		}
+		
+		LIBMTP_Get_Supported_Filetypes(device, &supportedFiletypes, &supportedFiletypesLength);
+    
+    [self downloadAlbumList];
+		
+		// download the data files to the cache
+		//[self dataFiles];
+	}
+	else
+	{
+		[self storeDeviceString];
+		[self storeFirmwareVersionString];
+		[self storeDeviceIDString];
+		[self storeDeviceVersionString];
+	}
+}
+
 @end
 
 // this is copied (as allowed) from http://theocacao.com/document.page/498
@@ -2465,7 +2945,6 @@ int mtp_progress(uint64_t const sent, uint64_t const total, void const * const d
                      fraction: 1.0];
     
     [newImage unlockFocus];
-  
   }
   
   return [newImage autorelease];
