@@ -1532,11 +1532,12 @@
     NSString * Type;
     NSString * Description;
     NSData * data = [frame getRawFrameData]; // extract the data from the frame
+    
     const char * charPtr = (const char *) [data bytes];
     char textCoding = * charPtr;  // the first byte is the text coding for the description
     int i,y;
     
-    for (i = 1 ; i < [data length] - 1; i++)  // find the end of the Description text 
+    for (i = 1; i < [data length] - 1; i++)  // find the end of the Description text 
     {
         if (charPtr[i] == '\0') break;
     }
@@ -1544,10 +1545,20 @@
     Type = [NSString stringWithCString:charPtr+1 length:i];
     
     i++;
+    if (i >= [data length])
+    {
+      NSLog(@"Invalid frame in getImageFrom3Frame; giving up");
+      return nil;
+    }
     // get the picture type byte and convert into a information string;
     pictureType = [self decodeImageType:charPtr[i]];
     // scan for the end of the description
     i++;
+    if (i >= [data length])
+    {
+      NSLog(@"Invalid frame in getImageFrom3Frame; giving up");
+      return nil;
+    }
     y=i;
     for (; i < [data length] - 2; i++)  // find the end of the Description text 
     {
@@ -1556,6 +1567,11 @@
     
     // create the description text string
     i++;
+    if (i >= [data length])
+    {
+      NSLog(@"Invalid frame in getImageFrom3Frame; giving up");
+      return nil;
+    }
     if (textCoding == 0)
     {
         Description = [NSString stringWithCString:(void *)charPtr length:i-y];
@@ -1563,11 +1579,23 @@
     else 
     {
         i++;
+        if (i >= [data length])
+        {
+          NSLog(@"Invalid frame in getImageFrom3Frame; giving up");
+          return nil;
+        }
         Description = [[[NSString alloc] initWithData:[NSData dataWithBytesNoCopy:(void *)charPtr+y length: i-y freeWhenDone:NO] encoding:NSUTF8StringEncoding] autorelease];  //this is not the right encoding I will fix later
     // get the image 
     }
     
     if (charPtr[i]=='\0') i++;
+    
+    if (i >= [data length])
+    {
+      NSLog(@"Invalid frame in getImageFrom3Frame; giving up");
+      return nil;
+    }
+    
     tempImage = [NSBitmapImageRep imageRepWithData:[NSData dataWithBytesNoCopy:(void *)charPtr+i length: [data length]-i freeWhenDone:NO]];
     if (tempImage == NULL ) return NULL;
     [tempDictionary setObject:tempImage forKey:@"Image"];
