@@ -26,6 +26,10 @@
 #include <iconv.h>
 #include "gphoto2-endian.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 /* PTP datalayer byteorder */
 
 #define PTP_DL_BE			0xF0
@@ -151,6 +155,7 @@ typedef struct _PTPIPHeader PTPIPHeader;
 
 /* Operation Codes */
 
+/* PTP v1.0 operation codes */
 #define PTP_OC_Undefined                0x1000
 #define PTP_OC_GetDeviceInfo            0x1001
 #define PTP_OC_OpenSession              0x1002
@@ -180,6 +185,16 @@ typedef struct _PTPIPHeader PTPIPHeader;
 #define PTP_OC_CopyObject               0x101A
 #define PTP_OC_GetPartialObject         0x101B
 #define PTP_OC_InitiateOpenCapture      0x101C
+/* PTP v1.1 operation codes */
+#define PTP_OC_StartEnumHandles		0x101D
+#define PTP_OC_EnumHandles		0x101E
+#define PTP_OC_StopEnumHandles		0x101F
+#define PTP_OC_GetVendorExtensionMaps	0x1020
+#define PTP_OC_GetVendorDeviceInfo	0x1021
+#define PTP_OC_GetResizedImageObject	0x1022
+#define PTP_OC_GetFilesystemManifest	0x1023
+#define PTP_OC_GetStreamInfo		0x1024
+#define PTP_OC_GetStream		0x1025
 
 /* Eastman Kodak extension Operation Codes */
 #define PTP_OC_EK_GetSerial		0x9003
@@ -330,12 +345,17 @@ typedef struct _PTPIPHeader PTPIPHeader;
 #define PTP_OC_NIKON_SetProfileData	0x9009
 #define PTP_OC_NIKON_AdvancedTransfer	0x9010
 #define PTP_OC_NIKON_GetFileInfoInBlock	0x9011
-#define PTP_OC_NIKON_Capture		0x90C0
-#define PTP_OC_NIKON_SetControlMode	0x90C2
-#define PTP_OC_NIKON_CurveDownload	0x90C5
-#define PTP_OC_NIKON_CurveUpload	0x90C6
-#define PTP_OC_NIKON_CheckEvent		0x90C7
-#define PTP_OC_NIKON_DeviceReady	0x90C8
+#define PTP_OC_NIKON_Capture		0x90C0	/* 1 param,   no data */
+#define PTP_OC_NIKON_AfDrive		0x90C1	/* no params, no data */
+#define PTP_OC_NIKON_SetControlMode	0x90C2	/* 1 param,   no data */
+#define PTP_OC_NIKON_DelImageSDRAM	0x90C3	/* no params, no data */
+#define PTP_OC_NIKON_CurveDownload	0x90C5	/* 1 param,   data in */
+#define PTP_OC_NIKON_CurveUpload	0x90C6	/* 1 param,   data out */
+#define PTP_OC_NIKON_CheckEvent		0x90C7	/* no params, data in */
+#define PTP_OC_NIKON_DeviceReady	0x90C8	/* no params, no data */
+#define PTP_OC_NIKON_SetPreWBData	0x90C9	/* 3 params,  data out */
+					/* 0x90CA */
+#define PTP_OC_NIKON_AfCaptureSDRAM	0x90CB	/* no params, no data */
 #define PTP_OC_NIKON_GetDevicePTPIPInfo	0x90E0
 
 /* Microsoft / MTP extension codes */
@@ -425,6 +445,7 @@ typedef struct _PTPIPHeader PTPIPHeader;
 
 /* Response Codes */
 
+/* PTP v1.0 response codes */
 #define PTP_RC_Undefined                0x2000
 #define PTP_RC_OK                       0x2001
 #define PTP_RC_GeneralError             0x2002
@@ -458,6 +479,10 @@ typedef struct _PTPIPHeader PTPIPHeader;
 #define PTP_RC_SessionAlreadyOpened     0x201E
 #define PTP_RC_TransactionCanceled      0x201F
 #define PTP_RC_SpecificationOfDestinationUnsupported            0x2020
+/* PTP v1.1 response codes */
+#define PTP_RC_InvalidEnumHandle	0x2021
+#define PTP_RC_NoStreamEnabled		0x2022
+#define PTP_RC_InvalidDataSet		0x2023
 
 /* Eastman Kodak extension Response Codes */
 #define PTP_RC_EK_FilenameRequired	0xA001
@@ -474,8 +499,10 @@ typedef struct _PTPIPHeader PTPIPHeader;
 #define PTP_RC_MTP_Invalid_ObjectProp_Value	0xA803
 #define PTP_RC_MTP_Invalid_ObjectReference	0xA804
 #define PTP_RC_MTP_Invalid_Dataset		0xA806
-#define PTP_RC_MTP_Specification_By_Group_Unsupported		0xA808
+#define PTP_RC_MTP_Specification_By_Group_Unsupported		0xA807
+#define PTP_RC_MTP_Specification_By_Depth_Unsupported		0xA808
 #define PTP_RC_MTP_Object_Too_Large		0xA809
+#define PTP_RC_MTP_ObjectProp_Not_Supported	0xA80A
 
 /* Microsoft Advanced Audio/Video Transfer response codes 
 (microsoft.com/AAVT 1.0) */
@@ -492,6 +519,7 @@ typedef struct _PTPIPHeader PTPIPHeader;
 #define PTP_ERROR_DATA_EXPECTED		0x02FE
 #define PTP_ERROR_RESP_EXPECTED		0x02FD
 #define PTP_ERROR_BADPARAM		0x02FC
+#define PTP_ERROR_CANCEL		0x02FB
 
 /* PTP Event Codes */
 
@@ -526,6 +554,11 @@ typedef struct _PTPIPHeader PTPIPHeader;
 /* Gets 1 parameter, objectid pointing to DPOF object */
 #define PTP_EC_Nikon_AdvancedTransfer		0xC103
 
+/* MTP Event codes */
+#define PTP_EC_MTP_ObjectPropChanged		0xC801
+#define PTP_EC_MTP_ObjectPropDescChanged	0xC802
+#define PTP_EC_MTP_ObjectReferencesChanged	0xC803
+
 /* constants for GetObjectHandles */
 #define PTP_GOH_ALL_STORAGE 0xffffffff
 #define PTP_GOH_ALL_FORMATS 0x00000000
@@ -654,6 +687,8 @@ typedef struct _PTPObjectInfo PTPObjectInfo;
 #define PTP_OFC_TIFF_IT				0x380E
 #define PTP_OFC_JP2				0x380F
 #define PTP_OFC_JPX				0x3810
+/* ptp v1.1 has only DNG new */
+#define PTP_OFC_DNG				0x3811
 /* Eastman Kodak extension ancillary format */
 #define PTP_OFC_EK_M3U				0xb002
 /* Canon extension */
@@ -713,7 +748,8 @@ typedef struct _PTPObjectInfo PTPObjectInfo;
 #define PTP_OFC_MTP_vCalendar1			0xbe02
 #define PTP_OFC_MTP_vCalendar2			0xbe03
 #define PTP_OFC_MTP_UndefinedWindowsExecutable	0xbe80
-
+#define PTP_OFC_MTP_MediaCast			0xbe81
+#define PTP_OFC_MTP_Section			0xbe82
 
 /* PTP Association Types */
 #define PTP_AT_Undefined			0x0000
@@ -773,14 +809,13 @@ union _PTPPropertyValue {
 typedef union _PTPPropertyValue PTPPropertyValue;
 
 /* Metadata lists for MTP operations */
-struct _MTPPropList {
+struct _MTPProperties {
 	uint16_t 	 	property;
 	uint16_t 	 	datatype;
 	uint32_t 	 	ObjectHandle;
 	PTPPropertyValue 	propval;
-	struct _MTPPropList 	*next;
 };
-typedef struct _MTPPropList MTPPropList;
+typedef struct _MTPProperties MTPProperties;
 
 struct _PTPPropDescRangeForm {
 	PTPPropertyValue 	MinimumValue;
@@ -973,6 +1008,7 @@ typedef struct _PTPCanon_Property {
 
 /* Device Properties Codes */
 
+/* PTP v1.0 property codes */
 #define PTP_DPC_Undefined		0x5000
 #define PTP_DPC_BatteryLevel		0x5001
 #define PTP_DPC_FunctionalMode		0x5002
@@ -1005,6 +1041,20 @@ typedef struct _PTPCanon_Property {
 #define PTP_DPC_UploadURL		0x501D
 #define PTP_DPC_Artist			0x501E
 #define PTP_DPC_CopyrightInfo		0x501F
+/* PTP v1.1 property codes */
+#define PTP_DPC_SupportedStreams	0x5020
+#define PTP_DPC_EnabledStreams		0x5021
+#define PTP_DPC_VideoFormat		0x5022
+#define PTP_DPC_VideoResolution		0x5023
+#define PTP_DPC_VideoQuality		0x5024
+#define PTP_DPC_VideoFrameRate		0x5025
+#define PTP_DPC_VideoContrast		0x5026
+#define PTP_DPC_VideoBrightness		0x5027
+#define PTP_DPC_AudioFormat		0x5028
+#define PTP_DPC_AudioBitrate		0x5029
+#define PTP_DPC_AudioSamplingRate	0x502A
+#define PTP_DPC_AudioBitPerSample	0x502B
+#define PTP_DPC_AudioVolume		0x502C
 
 /* Proprietary vendor extension device property mask */
 #define PTP_DPC_EXTENSION_MASK		0xF000
@@ -1064,7 +1114,14 @@ typedef struct _PTPCanon_Property {
 #define PTP_DPC_CANON_FocalLengthTele	0xD026
 #define PTP_DPC_CANON_FocalLengthWide	0xD027
 #define PTP_DPC_CANON_FocalLengthDenominator	0xD028
-#define PTP_DPC_CANON_CaptureTransferMode		0xD029
+#define PTP_DPC_CANON_CaptureTransferMode	0xD029
+#define CANON_TRANSFER_ENTIRE_IMAGE_TO_PC	0x0002
+#define CANON_TRANSFER_SAVE_THUMBNAIL_TO_DEVICE	0x0004
+#define CANON_TRANSFER_SAVE_IMAGE_TO_DEVICE	0x0008
+/* we use those values: */
+#define CANON_TRANSFER_MEMORY		(2|1)
+#define CANON_TRANSFER_CARD		(8|4|1)
+
 #define PTP_DPC_CANON_Zoom		0xD02A
 #define PTP_DPC_CANON_NamePrefix	0xD02B
 #define PTP_DPC_CANON_SizeQualityMode	0xD02C
@@ -1323,7 +1380,7 @@ typedef struct _PTPCanon_Property {
 #define PTP_OPC_DisplayName				0xDCE0
 #define PTP_OPC_BodyText				0xDCE1
 #define PTP_OPC_Subject					0xDCE2
-#define PTP_OPC_Prority					0xDCE3
+#define PTP_OPC_Priority				0xDCE3
 #define PTP_OPC_GivenName				0xDD00
 #define PTP_OPC_MiddleNames				0xDD01
 #define PTP_OPC_FamilyName				0xDD02
@@ -1473,6 +1530,8 @@ typedef uint16_t (* PTPIOSendData)	(PTPParams* params, PTPContainer* ptp,
 typedef uint16_t (* PTPIOGetResp)	(PTPParams* params, PTPContainer* resp);
 typedef uint16_t (* PTPIOGetData)	(PTPParams* params, PTPContainer* ptp,
 	                                 PTPDataHandler *putter);
+typedef uint16_t (* PTPIOCancelReq)	(PTPParams* params, uint32_t transaction_id);
+
 /* debug functions */
 typedef void (* PTPErrorFunc) (void *data, const char *format, va_list args)
 #if (__GNUC__ >= 3)
@@ -1497,6 +1556,7 @@ struct _PTPParams {
 	PTPIOGetData	getdata_func;
 	PTPIOGetResp	event_check;
 	PTPIOGetResp	event_wait;
+	PTPIOCancelReq	cancelreq_func;
 
 	/* Custom error and debug function */
 	PTPErrorFunc	error_func;
@@ -1514,7 +1574,8 @@ struct _PTPParams {
 	int		split_header_data;
 
 	/* PTP: MTP specific structure. */
-	MTPPropList	*proplist;
+	MTPProperties	*props;
+	int		nrofprops;
 
 	/* PTP: internal structures used by ptp driver */
 	PTPObjectHandles handles;
@@ -1561,6 +1622,7 @@ uint16_t ptp_usb_event_wait	(PTPParams* params, PTPContainer* event);
 uint16_t ptp_usb_control_get_extended_event_data (PTPParams *params, char *buffer, int *size);
 uint16_t ptp_usb_control_device_reset_request (PTPParams *params);
 uint16_t ptp_usb_control_get_device_status (PTPParams *params, char *buffer, int *size);
+uint16_t ptp_usb_control_cancel_request (PTPParams *params, uint32_t transid);
 
 
 int      ptp_ptpip_connect	(PTPParams* params, const char *port);
@@ -1636,10 +1698,10 @@ uint16_t ptp_mtp_setobjectpropvalue (PTPParams* params, uint32_t oid, uint16_t o
 				PTPPropertyValue *value, uint16_t datatype);
 uint16_t ptp_mtp_getobjectreferences (PTPParams* params, uint32_t handle, uint32_t** ohArray, uint32_t* arraylen);
 uint16_t ptp_mtp_setobjectreferences (PTPParams* params, uint32_t handle, uint32_t* ohArray, uint32_t arraylen);
-uint16_t ptp_mtp_getobjectproplist (PTPParams* params, uint32_t handle, MTPPropList **proplist);
+uint16_t ptp_mtp_getobjectproplist (PTPParams* params, uint32_t handle, MTPProperties **props, int *nrofprops);
 uint16_t ptp_mtp_sendobjectproplist (PTPParams* params, uint32_t* store, uint32_t* parenthandle, uint32_t* handle,
-				     uint16_t objecttype, uint64_t objectsize, MTPPropList *proplist);
-uint16_t ptp_mtp_setobjectproplist (PTPParams* params, MTPPropList *proplist);
+				     uint16_t objecttype, uint64_t objectsize, MTPProperties *props, int nrofprops);
+uint16_t ptp_mtp_setobjectproplist (PTPParams* params, MTPProperties *props, int nrofprops);
 
 /* Eastman Kodak extensions */
 uint16_t ptp_ek_9007 (PTPParams* params, unsigned char **serial, unsigned int *size);
@@ -1670,7 +1732,10 @@ uint16_t ptp_canon_endshootingmode (PTPParams* params);
 uint16_t ptp_canon_viewfinderon (PTPParams* params);
 uint16_t ptp_canon_viewfinderoff (PTPParams* params);
 
-uint16_t ptp_canon_aeafawb (PTPParams* params, uint32_t p1);
+#define PTP_CANON_RESET_AE	0x1
+#define PTP_CANON_RESET_AF	0x2
+#define PTP_CANON_RESET_AWB	0x4
+uint16_t ptp_canon_reset_aeafawb (PTPParams* params, uint32_t flags);
 uint16_t ptp_canon_checkevent (PTPParams* params, 
 				PTPUSBEventContainer* event, int* isevent);
 uint16_t ptp_canon_focuslock (PTPParams* params);
@@ -1699,7 +1764,7 @@ uint16_t ptp_canon_get_customize_data (PTPParams* params, uint32_t themenr,
 				unsigned char **data, unsigned int *size);
 uint16_t ptp_canon_getpairinginfo (PTPParams* params, uint32_t nr, unsigned char**, unsigned int*);
 
-uint16_t ptp_canon_eos_getstorageids (PTPParams* params);
+uint16_t ptp_canon_eos_getstorageids (PTPParams* params, PTPStorageIDs* storageids);
 uint16_t ptp_canon_eos_getstorageinfo (PTPParams* params, uint32_t p1);
 uint16_t ptp_canon_eos_getpartialobject (PTPParams* params, uint32_t oid, uint32_t off, uint32_t xsize, unsigned char**data);
 uint16_t ptp_canon_eos_setdevicepropvalueex (PTPParams* params, unsigned char* data, unsigned int size);
@@ -1720,6 +1785,7 @@ uint16_t ptp_nikon_writewifiprofile (PTPParams* params, PTPNIKONWifiProfile* pro
 uint16_t ptp_nikon_deletewifiprofile (PTPParams* params, uint32_t profilenr);
 uint16_t ptp_nikon_setcontrolmode (PTPParams* params, uint32_t mode);
 uint16_t ptp_nikon_capture (PTPParams* params, uint32_t x);
+uint16_t ptp_nikon_capture_sdram (PTPParams* params);
 uint16_t ptp_nikon_check_event (PTPParams* params, PTPUSBEventContainer **evt, int *evtcnt);
 uint16_t ptp_nikon_getfileinfoinblock (PTPParams* params, uint32_t p1, uint32_t p2, uint32_t p3,
 					unsigned char **data, unsigned int *size);
@@ -1749,7 +1815,18 @@ ptp_render_property_value(PTPParams* params, uint16_t dpc,
 int ptp_render_ofc(PTPParams* params, uint16_t ofc, int spaceleft, char *txt);
 int ptp_render_opcode(PTPParams* params, uint16_t opcode, int spaceleft, char *txt);
 int ptp_render_mtp_propname(uint16_t propid, int spaceleft, char *txt);
+MTPProperties *ptp_get_new_object_prop_entry(MTPProperties **props, int *nrofprops);
+void ptp_destroy_object_prop(MTPProperties *prop);
+void ptp_destroy_object_prop_list(MTPProperties *props, int nrofprops);
+MTPProperties *ptp_find_object_prop_in_cache(PTPParams *params, uint32_t const handle, uint32_t const attribute_id);
+void ptp_remove_object_from_cache(PTPParams *params, uint32_t handle);
+uint16_t ptp_add_object_to_cache(PTPParams *params, uint32_t handle);
 
 /* ptpip.c */
 void ptp_nikon_getptpipguid (unsigned char* guid);
+
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif /* __PTP_H__ */
